/******************************************************************************\
 *  _   ___     ____  __               _                                      *
 * | | | \ \___/ /  \/  | ___ __ _ ___| |_                                    *
 * | |_| |\     /| |\/| |/ __/ _` / __| __|                                   *
 * |  _  | \ - / | |  | | (_| (_| \__ \ |_                                    *
 * |_| |_|  \_/  |_|  |_|\___\__,_|___/\__|                                   *
 *                                                                            *
 * This file is part of the HAMcast project.                                  *
 *                                                                            *
 * HAMcast is free software: you can redistribute it and/or modify            *
 * it under the terms of the GNU Lesser General Public License as published   *
 * by the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                        *
 *                                                                            *
 * HAMcast is distributed in the hope that it will be useful,                 *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                       *
 * See the GNU Lesser General Public License for more details.                *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public License   *
 * along with HAMcast. If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * Contact: HAMcast support <hamcast-support@informatik.haw-hamburg.de>       *
\******************************************************************************/

#include "hamcast/hamcast_logging.h"
#include "hcDiscovery/Inet_Stuff/if_prop.hpp"
#include <arpa/inet.h>
#include <iostream>
#include <sstream>
#include <string>
#include <cstring>
#include <cstdio>
#include <errno.h>
#include <sys/ioctl.h>
#include <fstream>
#include <ifaddrs.h>

using namespace std;

const char* getS(int status) { //get Status
     HC_LOG_TRACE("");

     static string strue = "true";
     static string sfalse = "false";
     if (status) {
          return strue.c_str();
     } else {
          return sfalse.c_str();
     }
}

int hexChar_To_int(char x){
     HC_LOG_TRACE("");

     if(x >= '0' && x <='9'){
          return x - '0';
     }else if(x >= 'a' && x <='f'){
          return x - 'a' + 10;
     }else if(x >= 'A' && x <='F'){

          return x - 'A' + 10;
     }else{
          return -1;
     }
}

void if_prop::printIfInfo() {
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          cout << "printIfInfo:sck invalid" << endl;
          return;
     }
     cout << "--Network Interfaces--" << endl;
     cout << "found " << m_nInterfaces << " Interface[s]" << endl << endl;

     char* addr;

     /* Iterate through the list of interfaces. */
     for (int i = 0; i < m_nInterfaces; i++) {
          struct ifreq *item = &m_ifr[i];
          printf("%s\n", item->ifr_ifrn.ifrn_name);
          //print IPv4-addr
          addr = inet_ntoa(((struct sockaddr_in *) &item->ifr_addr)->sin_addr);
          printf("\tIP:\t\t%s\n", addr);
          //print IPv6 addr

          //print Mac
          if (getHwAddr(item)) { //load hardware address
               addr = (char*) &(item->ifr_ifru.ifru_hwaddr.sa_data);

               printf("addr Famalie %d\n", item->ifr_ifru.ifru_hwaddr.sa_family);

               printf("\tMAC:\t\t");
               for (int j = 0; j < IFHWADDRLEN; j++) {
                    printf("%2.2x ", (unsigned char) addr[j]);
               }
               printf("\n");
          }

          //print Falgs auswerten
          if (getFlags(item)) { //load flags
               printf("\tup:\t\t%s\n", getS(item->ifr_flags & IFF_UP));
               printf("\trunning:\t%s\n", getS(item->ifr_flags & IFF_RUNNING));
               printf("\tloopback:\t%s\n", getS(item->ifr_flags & IFF_LOOPBACK));
               printf("\tall multi:\t%s\n", getS(item->ifr_flags & IFF_ALLMULTI));
               printf("\tmulticast:\t%s\n", getS(item->ifr_flags & IFF_MULTICAST));
          }

          printf("\n");
     }
}

void if_prop::printJoinedGroup(){
     HC_LOG_TRACE("");

     //--filesystem access--
     MGroup_map::iterator item= m_mgroup_map.begin();

     while(item != m_mgroup_map.end()){
          cout << (*item).first << endl; //current device

          MGroup_value* tmp =&((*item).second);

          for(unsigned int i=0; i< tmp->size(); i++){ //set of ip addr
               cout << "\t" << (*tmp)[i] << endl;
          }

          item++;
     }
}

void if_prop::print_Address_list(){
     HC_LOG_TRACE("");

     //--getifaddrs access--
     struct ifaddrs* ifa=NULL,*ifEntry=NULL;
     void* addPtr = NULL;
     int rc = 0;
     char addressBuffer[INET6_ADDRSTRLEN];

     rc = getifaddrs(&ifa);
     if (rc==0) {
          for(ifEntry=ifa; ifEntry!=NULL; ifEntry=ifEntry->ifa_next) {
               if(ifEntry->ifa_addr->sa_data == NULL) {
                    cout << "no Data" << endl;
                    continue;
               }
               if(ifEntry->ifa_addr->sa_family==AF_INET) {
                    addPtr = &((struct sockaddr_in *)ifEntry->ifa_addr)->sin_addr;
               } else if(ifEntry->ifa_addr->sa_family==AF_INET6) {
                    addPtr = &((struct sockaddr_in6 *)ifEntry->ifa_addr)->sin6_addr;
               } else {
                    //It isn't IPv4 or IPv6
                    cout << "not AF_INET or AF_INET6" << endl;
                    continue;
               }

               const char *a = inet_ntop(ifEntry->ifa_addr->sa_family,
                                         addPtr,
                                         addressBuffer,
                                         sizeof(addressBuffer));
               if(a != NULL) {
                    cout << "ifname: " << ifEntry->ifa_name << endl;
                    cout << "\tip-addr: " << a << endl;
               }
          }
     }
     freeifaddrs(ifa);
}

if_prop::if_prop():
          m_sck(-1), m_ifr(0), m_nInterfaces(0), m_ifa(0)
{
     HC_LOG_TRACE("");
}

if_prop::~if_prop() {
     HC_LOG_TRACE("");

     if (is_sck_valid()) {
          close(m_sck);
     }

     if(is_ifa_valid()){
          freeifaddrs(m_ifa);
     }
}

bool if_prop::init_IfInfo() {
     HC_LOG_TRACE("");

     // Get a socket handle
     m_sck = socket(AF_INET, SOCK_DGRAM, 0);
     if (m_sck < 0) {
          HC_LOG_ERROR("failed to create Socket! Errno: " << errno);
          return false; // failed
     }

     // Query available interfaces
     m_ifc.ifc_len = sizeof(m_buf);
     m_ifc.ifc_buf = m_buf;

     return true;
}

bool if_prop::refresh_network_interfaces() {
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFCONF, &m_ifc) < 0) {
          HC_LOG_ERROR("failed to get Interface config! Errno: " << errno);
          close(m_sck);
          m_sck = -1;
          return false;
     }

     m_ifr = m_ifc.ifc_req;
     m_nInterfaces = m_ifc.ifc_len / sizeof(struct ifreq);

     //--getifaddrs access--
     if(is_ifa_valid()){
          freeifaddrs(m_ifa);
     }

     if(getifaddrs(&m_ifa) < 0){
          HC_LOG_ERROR("getifaddrs faild! Errno: " << errno );
          return false;
     }

     return true;
}


bool if_prop::getAddr(ifreq *item) { //load flags
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFADDR, item) == -1) {
          HC_LOG_ERROR("failed to get Addr! Errno: " << errno);
          return false;
     }
     return true;
}

bool if_prop::getFlags(ifreq *item) { //load flags
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFFLAGS, item) == -1) {
          HC_LOG_ERROR("failed to get flags! Errno: " << errno);
          return false;
     }
     return true;
}

bool if_prop::getHwAddr(ifreq *item) { //load hardware address
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFHWADDR, item) == -1) {
          HC_LOG_ERROR("failed to get hardware address! Errno: " << errno);
          return false;
     }
     return true;
}

bool if_prop::getMask(ifreq* item){
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFNETMASK, item) == -1) {
          HC_LOG_ERROR("failed to get mask! Errno: " << errno);
          return false;
     }

     return true;
}

bool if_prop::getBroadcast(ifreq* item){
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFBRDADDR, item) == -1) {
          HC_LOG_ERROR("failed to get broadcast address! Errno: " << errno);
          return false;
     }

     return true;
}

bool if_prop::getMTU(ifreq* item){
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return false;
     }

     if (ioctl(m_sck, SIOCGIFMTU, item) == -1) {
          HC_LOG_ERROR("failed to get mtu! Errno: " << errno);
          return false;
     }

     return true;
}


unsigned int if_prop::getInterfaceCount() {
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return -1;
     }

     return m_nInterfaces;
}

struct ifreq* if_prop::getInterfaceItem(unsigned int index) {
     HC_LOG_TRACE("");

     //--ioctl access--
     if (!is_sck_valid()) {
          HC_LOG_ERROR("sck invalid");
          return NULL;
     }

     if (index >= getInterfaceCount()) {
          HC_LOG_ERROR("wrong index: " << index);
          return NULL;
     }

     return &m_ifr[index];
}

string if_prop::convertMAC_Addr(ifreq* item){
     HC_LOG_TRACE("");

     //--general--
     char* addr;
     ostringstream str;
     char buffer[4];
     addr = (char*) &(item->ifr_ifru.ifru_hwaddr.sa_data);
     for (int j = 0; j < IFHWADDRLEN-1; j++) {
          snprintf(buffer,sizeof(buffer),"%2.2x:", (unsigned char) addr[j]);
          str << buffer;
     }
     snprintf(buffer,sizeof(buffer),"%2.2x", (unsigned char) addr[IFHWADDRLEN-1]);
     str << buffer;
     return str.str();
}

bool if_prop::refresh_joined_groups(){
     HC_LOG_TRACE("");

     //--filesystem access--
     m_mgroup_map.clear();

     ifstream file;
     char cstr[MAX_N_LINE_LENGTH];
     string tmp;
     string device;


     //--IPv4--
     //fill m_mgroup_map
     //file.open("/home/barbie/Dropbox/HAMcast/Recherche/igmp");
     file.open(JOINED_GROUP_PATH_V4);
     if(!file){
          HC_LOG_ERROR("can't open file: " << JOINED_GROUP_PATH_V4);
     }else{
          int status=0;  //0 = status line | 1 = counter line
          int nline=0;
          int ipCounter;
          MGroup_value ipAddr;

          file.getline(cstr,sizeof(cstr));
          while(!file.eof()){
               stringstream strline;
               strline << cstr;

               if(nline==0){
                    //init line nothing to do
               }else if(status == 0){
                    strline >> tmp; //Idx
                    strline >> device; //Device
                    strline >> tmp; //:
                    strline >> ipCounter; //Count
                    status=1;
               }else if(status == 1){
                    strline >> tmp; //Group
                    ipAddr.push_back(hexCharAddr_To_ipFormart(tmp,AF_INET));

                    ipCounter--;
                    if(ipCounter<=0){
                         status = 0;
                         m_mgroup_map.insert(MGroup_pair(device,ipAddr));
                         ipAddr.clear();
                    }
               }
               nline++;
               file.getline(cstr,sizeof(cstr));
          }
     }
     file.close();
     file.clear();

     //--IPv6--
     //fill m_mgroup_map
     //file.open("/home/barbie/Dropbox/HAMcast/Recherche/igmp6");
     file.open(JOINED_GROUP_PATH_V6);
     if(!file){
          HC_LOG_ERROR("can't open file: " << JOINED_GROUP_PATH_V6);
     }else{
          file.getline(cstr,sizeof(cstr));
          while(!file.eof()){
               stringstream strline;
               strline << cstr;

               strline >> tmp; //interface counter
               strline >> device; //Device

               MGroup_value* ipAddr= &m_mgroup_map[device];

               strline >> tmp; //ipv6 addr
               ipAddr->push_back(hexCharAddr_To_ipFormart(tmp,AF_INET6));

               file.getline(cstr,sizeof(cstr));
          }
          file.close();


     }

     return true;
}

string if_prop::hexCharAddr_To_ipFormart(string& ipAddr, int addrFamily){
     HC_LOG_TRACE("");

     //--general--
     ostringstream normal_ip;

     if(addrFamily == AF_INET){ //fill normal_ip
          //"FB0000E0" ==> "251.0.0.224"
          int z1, z2;
          for(unsigned int i=0; i< ipAddr.length()-2; i+=2){
               z1=hexChar_To_int(ipAddr.at(i));
               z2=hexChar_To_int(ipAddr.at(i+1));
               normal_ip <<(z1*16+z2) << ".";
          }

          z1=hexChar_To_int(ipAddr.at(ipAddr.length()-2));
          z2=hexChar_To_int(ipAddr.at(ipAddr.length()-1));
          normal_ip <<(z1*16+z2);
          return normal_ip.str();
     }else if(addrFamily == AF_INET6){ //fill normal ip
          //"ff020000000000000000000000000001" ==> "ff02:0000:0000:0000:0000:0000:0000:0001"
          for(unsigned int i = 0; i<ipAddr.length(); i++){
               normal_ip << ipAddr.at(i);
               if((i+1)%4 == 0 && i<ipAddr.length()-1){
                    normal_ip << ":";
               }
          }
          return normal_ip.str();
     }else{
          HC_LOG_ERROR("wrong addrFamily: " << addrFamily);
          return string();
     }

}

unsigned int if_prop::getJoinedGroupsCount(string& ifName){
     HC_LOG_TRACE("");

     //--filesystem access--
     MGroup_value* tmp = &m_mgroup_map[ifName];
     return tmp->size();
}

string if_prop::getJoinedGroup(string& ifName, unsigned int index){
     HC_LOG_TRACE("");

     //--filesystem access--
     MGroup_value* tmp = &m_mgroup_map[ifName];
     return (*tmp)[index];
     //return string();
}

vector<struct in6_addr*> if_prop::getIPv6Addr(string& ifName){
     HC_LOG_TRACE("");

     //--ioctl access--
     vector<struct in6_addr*> addV;

     if (!is_ifa_valid()) {
          HC_LOG_ERROR("ifa invalid");
          return addV;
     }

     struct ifaddrs* ifEntry=NULL;

     for(ifEntry=m_ifa; ifEntry!=NULL; ifEntry=ifEntry->ifa_next) {
          if(ifEntry->ifa_addr->sa_data != NULL &&
             ifEntry->ifa_addr->sa_family==AF_INET6 &&
             strcmp(ifName.c_str(), ifEntry->ifa_name)==0){
               addV.push_back(&((struct sockaddr_in6 *)ifEntry->ifa_addr)->sin6_addr);
          }
     }

     return addV;
}


/*
void if_prop::print_Address_list(){

    struct ifaddrs* ifa=NULL,*ifEntry=NULL;
    void* addPtr = NULL;
    int rc = 0;
    char addressBuffer[INET6_ADDRSTRLEN];

    rc = getifaddrs(&ifa);
    if (rc==0) {
        for(ifEntry=ifa; ifEntry!=NULL; ifEntry=ifEntry->ifa_next) {
            if(ifEntry->ifa_addr->sa_data == NULL) {
                cout << "no Data" << endl;
                continue;
            }
            if(ifEntry->ifa_addr->sa_family==AF_INET) {
                 addPtr = &((struct sockaddr_in *)ifEntry->ifa_addr)->sin_addr;
            } else if(ifEntry->ifa_addr->sa_family==AF_INET6) {
                 addPtr = &((struct sockaddr_in6 *)ifEntry->ifa_addr)->sin6_addr;
            } else {
                //It isn't IPv4 or IPv6
                cout << "not AF_INET or AF_INET6" << endl;
                continue;
            }

            const char *a = inet_ntop(ifEntry->ifa_addr->sa_family,
                          addPtr,
                          addressBuffer,
                          sizeof(addressBuffer));
            if(a != NULL) {
                cout << "ifname: " << ifEntry->ifa_name << endl;
                cout << "\tip-addr: " << a << endl;
            }
        }
    }
    freeifaddrs(ifa);
}

 */
