/******************************************************************************\
 *  _   ___     ____  __               _                                      *
 * | | | \ \___/ /  \/  | ___ __ _ ___| |_                                    *
 * | |_| |\     /| |\/| |/ __/ _` / __| __|                                   *
 * |  _  | \ - / | |  | | (_| (_| \__ \ |_                                    *
 * |_| |_|  \_/  |_|  |_|\___\__,_|___/\__|                                   *
 *                                                                            *
 * This file is part of the HAMcast project.                                  *
 *                                                                            *
 * HAMcast is free software: you can redistribute it and/or modify            *
 * it under the terms of the GNU Lesser General Public License as published   *
 * by the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                        *
 *                                                                            *
 * HAMcast is distributed in the hope that it will be useful,                 *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                       *
 * See the GNU Lesser General Public License for more details.                *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public License   *
 * along with HAMcast. If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * Contact: HAMcast support <hamcast-support@informatik.haw-hamburg.de>       *
\******************************************************************************/

#include "hamcast/hamcast_logging.h"
#include "hcDiscovery/DiscoveryTools/basic_dis.hpp"
#include "hcDiscovery/Inet_Stuff/if_prop.hpp"
#include "hcDiscovery/Inet_Stuff/mc_socket.hpp"
#include "hcDiscovery/hamcast_logging.h"
#include "hcDiscovery/Inet_Stuff/if_prop.hpp"
#include "hcDiscovery/Inet_Stuff/Protocol_Infos/ip_v6_tool.hpp"
#include "hcDiscovery/DiscoveryTools/mrinfo.hpp"
#include "hcDiscovery/Inet_Stuff/Protocol_Infos/test_output.hpp"
#include "hcDiscovery/service_discovery.hpp"

#include <iostream>
using namespace std;


void test_IfInfo();
void test_DiscoveryTankIfInfo();
void test_DiscoveryTankProto();
void test_DiscoveryMCSocket();
void test_DiscoveryInfoChain();
void test_service_discovery();
void test_DiscoveryLogTest();
void test_test();

/**
 * @brief Describes a list of key/value pairs.
 */
typedef struct hc_kvp_list_s
{
     const char* key;
     const char* value;
     struct hc_kvp_list_s* next;
}
hc_kvp_list_t;

/**
 * @brief add a new item to  @struct hc_kvp_list_t list
 */
hc_kvp_list_t* add_KVP_to_list(hc_kvp_list_t *kvp, const char* key, const char* value);

/**
 * @brief delete all items in the @struct hc_kvp_list_t list
 */
void del_KVP_list(hc_kvp_list_t *kvp);

string protoResolver(dis_proto_type i);
string ipAddrResolver(string ipAddr);

int main(void) {
     hc_set_default_log_fun(HC_LOG_TRACE_LVL);

     //test_DiscoveryMCSocket();
     //test_DiscoveryTankIfInfo();
     //test_DiscoveryTankProto();
     //test_DiscoveryInfoChain();
     test_service_discovery();
     //test_DiscoveryLogTest();
     //test_DiscoveryJoinBadGroups();
     //test_test();
     return 0;
}

void test_IfInfo(){
}

void test_DiscoveryTankIfInfo(){
     cout <<"############################" << endl;
     cout <<"##test_DiscoveryTankIfInfo##" << endl;
     cout <<"############################" << endl << endl;
     dis_tank* dt=dis_tank::getInstance();
     dis_tank_item* dtItem;
     //DiscoveryProtocol* dtProt;
     if(dt->init_DiscoveryTank()){
          if(dt->refreshTankFast()){
               if(dt->refreshJoinedGroups()){
                    cout<< "anzahl Interfaces: " << dt->getIfCount() << endl;
                    for (unsigned int i=0; i < dt->getIfCount(); i++) {
                         dtItem = dt->getIf(i);
                         cout << "##-- "<< dtItem->getName() << " <IfIndex=" << dtItem->getIfIndex() << ">" <<" --##" << endl;
                         cout << "\t-IPv4: " << dtItem->getIPv4_Addr() << endl;
                         cout << "\t-Mask: " << dtItem->getMask() << endl;
                         cout << "\t-Broadcast: " << dtItem->getBroadcast() << endl;

                         vector<string> ipv6Addr = dtItem->getIPv6_Addr();
                         cout << "\t-IPv6 [" << ipv6Addr.size() <<"]" << endl;
                         for(unsigned int i=0;i<ipv6Addr.size();i++){
                              cout << "\t\t" << ipv6Addr[i] << endl;
                         }
                         cout << "\t-MAC: " << dtItem->getMAC_Addr() << endl;
                         cout << "\t-isUP: " << dtItem->isUP() << endl;
                         cout << "\t-isRunning: " << dtItem->isRunning() << endl;
                         cout << "\t-isLoopback: " << dtItem->isLoopback() << endl;
                         cout << "\t-isPointToPoint: " << dtItem->isPointToPoint() << endl;
                         cout << "\t-isALLMulti: " << dtItem->isAllMulti() << endl;
                         cout << "\t-hasMulticast: " << dtItem->hasMulticast() << endl;
                         cout << "\t-MTU: " << dtItem->getMTU() << endl;
                         cout << "\t-Speed: " << dtItem->getNetworkSpeed() << endl;

                         unsigned int groupCount = dtItem->getJoinedGroupCount();
                         cout << "\t-joined groups [" << groupCount << "]" << endl;
                         for(unsigned int j=0; j < groupCount; j++)     {
                              cout << "\t\t" << dtItem->getJoinedGroup(j) << endl;
                              string tmp = ipAddrResolver(dtItem->getJoinedGroup(j));
                              if (!tmp.empty()){
                                   cout << "\t\t\t- " << tmp << endl;
                              }
                         }
                    }
               }
          }else{
               cout << "refreshTankFast fehlgeschlagen" << endl;
          }
     }else{
          cout << "init_DiscoveryTank fehlgeschlagen" << endl;
     }

}

void test_DiscoveryTankProto(){
     cout <<"###########################" << endl;
     cout <<"##test_DiscoveryTankProto##" << endl;
     cout <<"###########################" << endl << endl;
     dis_tank* dt=dis_tank::getInstance();
     dis_tank_item* dtItem;
     dis_proto* dtProt;
     int count=0;

     if(dt->init_DiscoveryTank()){
          //while(1){
          if(dt->refreshTankFast()){
               cout<< "##-- Interfaces (" << dt->getIfCount() << ") --## counter= " << count++ << endl;
               for (unsigned int i=0; i <dt->getIfCount() ; i++) {
                    dtItem = dt->getIf(i);

                    if(dtItem){
                         cout << "##-- "<< dtItem->getName()<<" --##" << endl;

                         dtItem->refreshProtocolFast();

                         cout << "\t-- Protokolle (" << dtItem->getProtocolCount() << ") --" << endl;
                         for (unsigned int j = 0; j < dtItem->getProtocolCount(); j++) {
                              dtProt = dtItem->getProtocol(j);
                              cout << "\t\t----" << endl;
                              cout << "\t\tType: " << protoResolver(dtProt->getType()) << endl;
                              cout << "\t\tHostAddr: " << dtProt->getHostAddr() << endl;
                              HC_LOG_DEBUG("counter: "<< count <<"; Type: "<< protoResolver(dtProt->getType()) << "; HostAddr: " << dtProt->getHostAddr());
                         }

                         cout << "\t-mrinfo: >>" << endl;
                         if(!dtItem->print_mrinfo()){
                              cout << "\t\tnot found" << endl;
                         }

                    }else{
                         cout << "Device nicht gefunden" << endl;
                    }
               }
          }else{
               cout << "refreshTankFast fehlgeschlagen" << endl;
          }
     }else{
          cout << "init_DiscoveryTank fehlgeschlagen" << endl;
     }
}

void test_DiscoveryMCSocket(){
     int sleepTime = 2;
     mc_socket m;
     cout << "-- Join and leave ipv4 --" << endl;
     m.create_UDP_IPv4_Socket();
     if(m.joinGroup("238.99.99.99","192.168.1.100")){
          cout << "join OK" << endl;
     }else{
          cout << "join failed" << endl;
     }
     sleep(sleepTime);
     if(m.leaveGroup("238.99.99.99","192.168.1.100")){
          cout << "leave OK" << endl;
     }else{
          cout << "leave failed" << endl;
     }
     sleep(sleepTime);

     cout << "-- Join and leave ipv6 --" << endl;
     m.create_UDP_IPv6_Socket();
     if(m.joinGroup("FF02:0:0:0:99:99:99:99","eth0" )){
          cout << "join OK" << endl;
     }else{
          cout << "join failed" << endl;
     }
     sleep(sleepTime);
     if(m.leaveGroup("FF02:0:0:0:99:99:99:99","eth0")){
          cout << "leave OK" << endl;
     }else{
          cout << "leave failed" << endl;
     }

     sleep(sleepTime);
     cout << "-- send Data IPv4 --" << endl;
     m.create_UDP_IPv4_Socket();
     m.chooseIf("192.168.1.100");
     if(m.send_UDP_Data("238.99.99.99","Hallo",9845)){
          cout << "Hallo an addr:238.99.99.99 mit port 9845 gesendet" << endl;
     }else{
          cout << "Fehler beim senden" << endl;
     }

     sleep(sleepTime);

     cout << "-- send Data IPv6 --" << endl;
     m.create_UDP_IPv6_Socket();
     m.chooseIf("eth0");
     if(m.send_UDP_Data("FF02:0:0:0:99:99:99:99","Hallo", 9845)){
          cout << "Hallo an addr:FF02:0:0:0:99:99:99:99 mit port 9845 gesendet";
     }else{
          cout << "Fehler beim senden" << endl;
     }

}

void test_DiscoveryLogTest(){
     hc_set_default_log_fun(HC_LOG_TRACE_LVL);
     HC_LOG_TRACE("");

     HC_LOG_DEBUG("HC_LOG_DEBUG");
     HC_LOG_INFO("HC_LOG_INFO");
     HC_LOG_WARN("HC_LOG_WARN");
     HC_LOG_ERROR("HC_LOG_ERROR");
     HC_LOG_FATAL("HC_LOG_FATAL");
}

/**
 * @brief key-value pairs of the service discovery
 *
 * @if IGMPv2 is found
 *    key: IGMP, value: 2
 *    key: Querier IGMP, value: IPv4 addr
 * @if IGMPv3 is found
 *    key: IGMP, value: 3
 *    key: Querier IGMP, value: IPv4 addr
 * @if MLDv1 is found
 *    key: MLD, value: 1
 *    key: Querier MLD, value: IPv6 addr
 * @if MLDv2 is found
 *    key: MLD, value: 2
 *    key: Querier MLD, value: IPv6 addr
 * @if PIMv1 is found
 *    key: PIM, value: 1
 * @if PIMv2 is found
 *    key: PIM, value: 2
 * @if return an Error
 *    key: ERROR, value: ERROR
 */

void test_DiscoveryInfoChain(){
     dis_tank* dt=dis_tank::getInstance();
     dis_tank_item* dtItem;
     dis_proto* dtProt;

     hc_kvp_list_t tech;
     tech.next=0;

     hc_kvp_list_t* list= &tech; //!!!!! name

     if(dt->init_DiscoveryTank() && dt->refreshTankFast()){
          dtItem = dt->getIf("eth0");//!!!!! insert your interface
          if(dtItem){

               dtItem->refreshProtocolFast();

               for (unsigned int j = 0; j < dtItem->getProtocolCount(); j++) {
                    dtProt = dtItem->getProtocol(j);
                    switch(dtProt->getType()){
                    case IGMPv2:
                         list = add_KVP_to_list(list,"IGMP","2");
                         list = add_KVP_to_list(list,"Querier IGMP", dtProt->getHostAddr().c_str());
                         break;
                    case IGMPv3:
                         list = add_KVP_to_list(list,"IGMP","3");
                         list = add_KVP_to_list(list,"Querier IGMP", dtProt->getHostAddr().c_str());
                         break;
                    case MLDv1:
                         list = add_KVP_to_list(list,"MLD","1");
                         list = add_KVP_to_list(list,"Querier MLD", dtProt->getHostAddr().c_str());
                         break;
                    case MLDv2:
                         list = add_KVP_to_list(list,"MLD","2");
                         list = add_KVP_to_list(list,"Querier MLD", dtProt->getHostAddr().c_str());
                         break;
                    case PIMv1:
                         list = add_KVP_to_list(list,"PIM","1");
                         break;
                    case PIMv2:
                         list = add_KVP_to_list(list,"PIM","2");
                         break;
                    default:
                         list = add_KVP_to_list(list,"ERROR","ERROR");
                    }
               }
          }else{
               cout << "DiscoveryTank wrong interface" << endl;
          }
     }else{
          cout << "DiscoveryTank Error" << endl;
     }


     hc_kvp_list_t* tmp = &tech;
     while(tmp->next){
          cout << "key: "<< tmp->next->key << " value: " << tmp->next->value << endl;
          tmp=tmp->next;
     }

     del_KVP_list(&tech);

}

void test_service_discovery(){
     service_discovery sd;
     sd.testoutput_service_discovery();
}

hc_kvp_list_t* add_KVP_to_list(hc_kvp_list_t *kvp, const char* key, const char* value){
     hc_kvp_list_t* tmp= new hc_kvp_list_t;
     tmp->key = key;
     tmp->value = value;
     tmp->next = 0;

     kvp->next = tmp;

     return tmp;
}

void del_KVP_list(hc_kvp_list_t *kvp){
     hc_kvp_list_t* tmp = kvp->next;
     hc_kvp_list_t* tmp_old = NULL;

     while(tmp){
          tmp_old = tmp;
          tmp = tmp->next;

          delete tmp_old;
     }
}

string protoResolver(dis_proto_type i){
     switch(i){
     case INIT_PROTO: return "INIT_PROTO";
     case IGMPv2: return "IGMPv2";
     case IGMPv3: return "IGMPv3";
     case MLDv2: return "MLDv2";
     case PIMv1: return "PIMv1";
     case PIMv2: return "PIMv2";
     case NTPv1: return "NTPv1";
     case NTPv2: return "NTPv2";

     default: return "ERROR";
     }
}

string ipAddrResolver(string ipAddr){
     string str[][2]={
          {IPV6_ALL_NODES_ADDR,"IPV6_ALL_NODES_ADDR"},
          {IPV6_ALL_LINK_LOCAL_ROUTER, "IPV6_ALL_LINK_LOCAL_ROUTER"},
          {IPV6_ALL_SITE_LOCAL_ROUTER,"IPV6_ALL_SITE_LOCAL_ROUTER"},
          {IPV6_ALL_PIM_ROUTERS, "IPV6_ALL_PIM_ROUTERS"},
          {IPV4_IGMPV3_ADDR, "IPV4_IGMPV3_ADDR"},
          {IPV4_ALL_HOST_ADDR,"IPV4_ALL_HOST_ADDR"},
          {IPV4_ALL_ROUTERS_ADDR, "IPV4_ALL_ROUTERS_ADDR"},
          {IPV4_PIMv2_ADDR,"IPV4_PIMv2_ADDR"},
          {IPV6_ALL_MLDv2_CAPABLE_ROUTERS, "IPV6_ALL_MLDv2_CAPABLE_ROUTERS"}
     };

     unsigned int nCount = 9;

     for(unsigned int i=0; i< nCount; i++){
          if(ip_v6_tool::equalAddr(ipAddr, str[i][0])){
               return str[i][1];
          }
     }

     return string();
}

void test_test(){
     /*mc_socket m;

     cout << "-- Join and leave ipv6 --" << endl;
     m.create_UDP_IPv6_Socket();
     if(m.joinGroup("FF02::99","eth0" )){
          cout << "join OK" << endl;
     }else{
          cout << "join failed" << endl;
     }
     sleep(5);
     if(m.leaveGroup("FF02::99","eth0")){
          cout << "leave OK" << endl;
     }else{
     */
}
